@@ -52,7 +52,15 @@ case $# in
      ;;
 esac
 
+maybe_activate_venv() {
+  if [ ! -z "${NANOARROW_PYTHON_VENV}" ]; then
+    source "${NANOARROW_PYTHON_VENV}/bin/activate"
+  fi
+}
+
 main() {
+   maybe_activate_venv
+
    pushd "${TARGET_NANOARROW_DIR}"
 
    # Clean the previous build
@@ -64,25 +72,27 @@ main() {
    doxygen
    popd
 
-   show_header "Run Doxygen for IPC extension"
-   pushd extensions/nanoarrow_ipc/src/apidoc
-   doxygen
-   popd
+   show_header "Build nanoarrow Python"
 
-   show_header "Run Doxygen for device extension"
-   pushd extensions/nanoarrow_device/src/apidoc
-   doxygen
+   pushd python
+   pip install .
    popd
 
-   pushd docs
-
    show_header "Build Sphinx project"
+   pushd docs
+
 
    # Use the README as the docs homepage
    pandoc ../README.md --from markdown --to rst -s -o source/README_generated.rst
 
+   # Use R README as the getting started guide for R
+   pandoc ../r/README.md --from markdown --to rst -s -o source/getting-started/r.rst
+
+   # Use Python README as the getting started guide for Python
+   pandoc ../python/README.md --from markdown --to rst -s -o source/getting-started/python.rst
+
    # Do some Markdown -> reST conversion
-   for f in source/*.md; do
+   for f in $(find source -name "*.md"); do
      fout=$(echo $f | sed -e s/.md/.rst/)
      pandoc $f --from markdown --to rst -s -o $fout
    done
@@ -96,7 +106,7 @@ main() {
    R CMD INSTALL ../r --preclean
 
    # Build R documentation
-   Rscript -e 'pkgdown::build_site_github_pages("../r", dest_dir = "../docs/_build/html/r", new_process = FALSE, install = FALSE)'
+   Rscript -e 'pkgdown::build_site("../r", override = list(destination = "../docs/_build/html/r"), new_process = FALSE, install = FALSE)'
 
    popd
 
