@@ -70,7 +70,7 @@ main() {
        --skip-existing
 
     header "Adding release notes"
-    local -r release_notes=$(python3 ${source_dir}/changelog.py)
+    local -r release_notes=$(python3 ${source_dir}/release_tools.py changelog)
     echo "${release_notes}"
     gh release edit \
        "${tag}" \
@@ -80,9 +80,6 @@ main() {
     header "Upload signatures for source"
     upload_asset_signatures "${tag}" $(find "${download_dir}" -type f \( -name 'apache-arrow-nanoarrow-*.tar.gz' \))
 
-    header "Upload signatures for R"
-    upload_asset_signatures "${tag}" $(find "${download_dir}" -type f \( -name 'nanoarrow_*.tar.gz' \))
-
     header "Upload signatures for docs"
     upload_asset_signatures "${tag}" "${download_dir}/docs.tgz"
 
