@@ -38,9 +38,9 @@ main() {
 
     rc_id="apache-arrow-nanoarrow-${version}-rc${rc_number}"
     release_id="apache-arrow-nanoarrow-${version}"
-    echo "Copying dev/ to release/"
+    echo "Moving dev/ to release/"
     svn \
-        cp \
+        mv \
         --username=$APACHE_USERNAME \
         -m "Apache Arrow nanoarrow ${version}" \
         https://dist.apache.org/repos/dist/dev/arrow/${rc_id} \
@@ -51,7 +51,7 @@ main() {
 
     echo "Success! The release is available here:"
     echo "  https://dist.apache.org/repos/dist/release/arrow/${release_id}"
-    echo "Please push the tag apache-arrow-nanoarrow-${version}!"
+    echo "git push upstream apache-arrow-nanoarrow-${version} before continuing!"
 }
 
 main "$@"
