@@ -20,8 +20,9 @@
 # Requirements
 # - cmake >= 3.14
 # - R >= 3.5.0
-# - Arrow C++ >= 9.0.0
 # - Python >= 3.8
+# - gpg (for key verification)
+# - shasum or sha512sum
 #
 # Environment Variables
 # - CMAKE_BIN: Command to use for cmake (e.g., cmake3 on Centos7)
@@ -243,20 +244,12 @@ test_cmake_project() {
 
 test_c() {
   show_header "Build and test C library"
-  test_cmake_project build . -DNANOARROW_BUILD_TESTS=ON
-
-  show_header "Build and test C IPC extension"
-  test_cmake_project build-ipc extensions/nanoarrow_ipc -DNANOARROW_IPC_BUILD_TESTS=ON
+  test_cmake_project build . -DNANOARROW_BUILD_TESTS=ON -DNANOARROW_IPC=ON
 }
 
 test_c_bundled() {
   show_header "Build test bundled C library"
-  test_cmake_project build-bundled . -DNANOARROW_BUILD_TESTS=ON -DNANOARROW_BUNDLE=ON
-
-  show_header "Build and test bundled C IPC extension"
-  test_cmake_project build-ipc extensions/nanoarrow_ipc \
-    -DNANOARROW_IPC_BUILD_TESTS=ON \
-    -DNANOARROW_IPC_BUNDLE=ON
+  test_cmake_project build-bundled . -DNANOARROW_BUILD_TESTS=ON -DNANOARROW_IPC=ON -DNANOARROW_BUNDLE=ON
 }
 
 test_r() {
@@ -277,11 +270,12 @@ test_r() {
 
   show_info "Build the R package source tarball"
 
-  # Running R CMD INSTALL on the R source directory is the most reliable cross-platform
-  # method to ensure the proper version of nanoarrow is vendored into the R package.
-  # Do this in a temporary library so not to overwrite the a user's existing package.
+  # Run bootstrap.R from within the r subdirectory
+  pushd r
+  "$R_BIN" -e 'source("bootstrap.R", echo = TRUE)'
+  popd
+
   mkdir "$NANOARROW_TMPDIR/tmplib"
-  "$R_BIN" CMD INSTALL r --preclean --library="$NANOARROW_TMPDIR/tmplib"
 
   # Builds the R source tarball
   pushd $NANOARROW_TMPDIR
@@ -354,6 +348,30 @@ test_python() {
   show_info "Testing wheel"
   python -m pytest -vv
 
+  if [ ${TEST_WITH_MEMCHECK} -gt 0 ]; then
+    show_info "Setup environment for Python/valgrind"
+
+    # To have more control over exactly which dependencies are available,
+    # set up a separate venv to run. Essentially, we check for leaks in
+    # all self-contained tests but not tests involving pyarrow or numpy.
+    VALGRIND_PYTHON_VENV="${NANOARROW_TMPDIR}/python/valgrind_venv"
+    python -m venv "${VALGRIND_PYTHON_VENV}"
+    source "${VALGRIND_PYTHON_VENV}/bin/activate"
+    python -m pip install --upgrade pip
+    python -m pip install pytest
+
+    # Ensure we have debug symbols for the build we pass to valgrind
+    export NANOARROW_DEBUG_EXTENSION=1
+    pip install .
+
+    show_info "Run Python tests with valgrind"
+    valgrind --tool=memcheck --leak-check=full --error-exitcode=1 \
+        --suppressions="${NANOARROW_SOURCE_DIR}/valgrind.supp" \
+        python -m pytest -vv
+
+    deactivate
+  fi
+
   popd
 }
 
