@@ -66,8 +66,6 @@ cpp_id_attributes = ["NANOARROW_CHECK_PRINTF_ATTRIBUTE"]
 # Breathe configuration
 breathe_projects = {
     "nanoarrow_c": "../../src/apidoc/xml",
-    "nanoarrow_ipc": "../../extensions/nanoarrow_ipc/src/apidoc/xml",
-    "nanoarrow_device": "../../extensions/nanoarrow_device/src/apidoc/xml",
 }
 breathe_default_project = "nanoarrow_c"
 
@@ -96,6 +94,7 @@ html_theme_options = {
         "version_match": get_version(),
     },
     "navbar_start": ["navbar-logo", "version-switcher"],
+    "navigation_with_keys": False,
 }
 
 html_context = {
