@@ -17,16 +17,12 @@
 
 import itertools
 from functools import cached_property
-from typing import Iterable, Tuple
+from typing import Iterable, Tuple, Union
 
-from nanoarrow._lib import (
-    DEVICE_CPU,
-    CArray,
-    CArrayView,
-    CBuffer,
-    CMaterializedArrayStream,
-    Device,
-)
+from nanoarrow._array import CArray, CArrayView
+from nanoarrow._array_stream import CMaterializedArrayStream
+from nanoarrow._buffer import CBufferView
+from nanoarrow._device import DEVICE_CPU, Device
 from nanoarrow.c_array import c_array, c_array_view
 from nanoarrow.c_array_stream import c_array_stream
 from nanoarrow.c_schema import c_schema
@@ -120,7 +116,7 @@ class Array(ArrayViewVisitable):
     the Arrow C Stream interface.
 
     Note that an :class:`Array` is not necessarily contiguous in memory (i.e.,
-    it may consist of zero or more ``ArrowArray``s).
+    it may consist of zero or more ``ArrowArray`` objects).
 
     Parameters
     ----------
@@ -295,7 +291,7 @@ class Array(ArrayViewVisitable):
         self._assert_one_chunk("scalar offset")
         return self._data.array(0).offset
 
-    def buffer(self, i: int) -> CBuffer:
+    def buffer(self, i: int) -> CBufferView:
         """Access a single buffer of a contiguous array
 
         Examples
@@ -304,12 +300,12 @@ class Array(ArrayViewVisitable):
         >>> import nanoarrow as na
         >>> array = na.Array([1, 2, 3], na.int32())
         >>> array.buffer(1)
-        nanoarrow.c_lib.CBufferView(int32[12 b] 1 2 3)
+        nanoarrow.c_buffer.CBufferView(int32[12 b] 1 2 3)
         """
         return self.buffers[i]
 
     @cached_property
-    def buffers(self) -> Tuple[CBuffer, ...]:
+    def buffers(self) -> Tuple[CBufferView, ...]:
         """Access buffers of a contiguous array.
 
         Examples
@@ -319,8 +315,8 @@ class Array(ArrayViewVisitable):
         >>> array = na.Array([1, 2, 3], na.int32())
         >>> for buffer in array.buffers:
         ...     print(buffer)
-        nanoarrow.c_lib.CBufferView(bool[0 b] )
-        nanoarrow.c_lib.CBufferView(int32[12 b] 1 2 3)
+        nanoarrow.c_buffer.CBufferView(bool[0 b] )
+        nanoarrow.c_buffer.CBufferView(int32[12 b] 1 2 3)
         """
         view = c_array_view(self)
         return tuple(view.buffers)
@@ -340,8 +336,8 @@ class Array(ArrayViewVisitable):
         ...     print(validity)
         ...     print(data)
         0 3
-        nanoarrow.c_lib.CBufferView(bool[0 b] )
-        nanoarrow.c_lib.CBufferView(int32[12 b] 1 2 3)
+        nanoarrow.c_buffer.CBufferView(bool[0 b] )
+        nanoarrow.c_buffer.CBufferView(int32[12 b] 1 2 3)
         """
         return iter_array_views(self)
 
@@ -546,6 +542,29 @@ class Array(ArrayViewVisitable):
             "to iterate over elements of this Array"
         )
 
+    def serialize(self, dst=None) -> Union[bytes, None]:
+        """Write this Array into dst as zero or more encapsulated IPC messages
+
+        Parameters
+        ----------
+        dst : file-like, optional
+            If present, a file-like object into which the chunks of this array
+            should be serialized. If omitted, this will create a ``io.BytesIO()``
+            and return the serialized result.
+        """
+        from nanoarrow.ipc import StreamWriter
+
+        if dst is None:
+            import io
+
+            with io.BytesIO() as dst:
+                writer = StreamWriter.from_writable(dst)
+                writer.write_stream(self, write_schema=False)
+                return dst.getvalue()
+        else:
+            writer = StreamWriter.from_writable(dst)
+            writer.write_stream(self, write_schema=False)
+
     def to_string(self, width_hint=80, items_hint=10) -> str:
         cls_name = _repr_utils.make_class_label(self, module="nanoarrow")
         len_text = f"[{len(self)}]"
