@@ -20,11 +20,14 @@ from functools import cached_property
 from itertools import islice, repeat
 from typing import Iterable, Tuple
 
-from nanoarrow._lib import CArrayView, CArrowType
+from nanoarrow._array import CArrayView
 from nanoarrow.c_array_stream import c_array_stream
 from nanoarrow.c_schema import c_schema, c_schema_view
+from nanoarrow.extension import resolve_extension
 from nanoarrow.schema import Schema
 
+from nanoarrow import _types
+
 
 def iter_py(obj, schema=None) -> Iterable:
     """Iterate over items in zero or more arrays
@@ -181,6 +184,8 @@ class PyIterator(ArrayViewBaseIterator):
 
     def __init__(self, schema, *, array_view=None):
         super().__init__(schema, array_view=array_view)
+        self._ext = resolve_extension(self._schema_view)
+        self._ext_params = self._ext.get_params(schema) if self._ext else None
 
         self._children = list(
             map(self._make_child, self._schema.children, self._array_view.children)
@@ -206,11 +211,15 @@ class PyIterator(ArrayViewBaseIterator):
 
     def _iter_chunk(self, offset, length):
         """Iterate over all elements in a slice of the current chunk"""
+
         # Check for an extension type first since this isn't reflected by
-        # self._schema_view.type_id. Currently we just return the storage
-        # iterator with a warning for extension types.
+        # self._schema_view.type_id.
         maybe_extension_name = self._schema_view.extension_name
-        if maybe_extension_name:
+        if self._ext:
+            maybe_iter = self._ext.get_pyiter(self, offset, length)
+            if maybe_iter:
+                return maybe_iter
+        elif maybe_extension_name:
             self._warn(
                 f"Converting unregistered extension '{maybe_extension_name}' "
                 "as storage type",
@@ -320,6 +329,12 @@ class PyIterator(ArrayViewBaseIterator):
             for start, end in zip(starts, ends):
                 yield bytes(data[start:end])
 
+    def _binary_view_iter(self, offset, length):
+        return self._array_view._iter_bytes(offset, length)
+
+    def _string_view_iter(self, offset, length):
+        return self._array_view._iter_str(offset, length)
+
     def _decimal_iter(self, offset, length):
         from decimal import Context, Decimal
         from sys import byteorder
@@ -353,7 +368,7 @@ class PyIterator(ArrayViewBaseIterator):
         storage = self._primitive_iter(offset, length)
         epoch = date(1970, 1, 1)
 
-        if self._schema_view.type_id == CArrowType.DATE32:
+        if self._schema_view.type_id == _types.DATE32:
             for item in storage:
                 if item is None:
                     yield item
@@ -544,24 +559,26 @@ def _get_tzinfo(tz_string, strategy=None):
 
 
 _ITEMS_ITER_LOOKUP = {
-    CArrowType.NA: "_null_iter",
-    CArrowType.BINARY: "_binary_iter",
-    CArrowType.LARGE_BINARY: "_binary_iter",
-    CArrowType.STRING: "_string_iter",
-    CArrowType.LARGE_STRING: "_string_iter",
-    CArrowType.STRUCT: "_struct_iter",
-    CArrowType.LIST: "_list_iter",
-    CArrowType.LARGE_LIST: "_list_iter",
-    CArrowType.FIXED_SIZE_LIST: "_fixed_size_list_iter",
-    CArrowType.DICTIONARY: "_dictionary_iter",
-    CArrowType.DATE32: "_date_iter",
-    CArrowType.DATE64: "_date_iter",
-    CArrowType.TIME32: "_time_iter",
-    CArrowType.TIME64: "_time_iter",
-    CArrowType.TIMESTAMP: "_timestamp_iter",
-    CArrowType.DURATION: "_duration_iter",
-    CArrowType.DECIMAL128: "_decimal_iter",
-    CArrowType.DECIMAL256: "_decimal_iter",
+    _types.NA: "_null_iter",
+    _types.BINARY: "_binary_iter",
+    _types.LARGE_BINARY: "_binary_iter",
+    _types.STRING: "_string_iter",
+    _types.LARGE_STRING: "_string_iter",
+    _types.STRUCT: "_struct_iter",
+    _types.LIST: "_list_iter",
+    _types.LARGE_LIST: "_list_iter",
+    _types.FIXED_SIZE_LIST: "_fixed_size_list_iter",
+    _types.DICTIONARY: "_dictionary_iter",
+    _types.DATE32: "_date_iter",
+    _types.DATE64: "_date_iter",
+    _types.TIME32: "_time_iter",
+    _types.TIME64: "_time_iter",
+    _types.TIMESTAMP: "_timestamp_iter",
+    _types.DURATION: "_duration_iter",
+    _types.DECIMAL128: "_decimal_iter",
+    _types.DECIMAL256: "_decimal_iter",
+    _types.STRING_VIEW: "_string_view_iter",
+    _types.BINARY_VIEW: "_binary_view_iter",
 }
 
 _PRIMITIVE_TYPE_NAMES = [
@@ -584,5 +601,5 @@ _PRIMITIVE_TYPE_NAMES = [
 ]
 
 for type_name in _PRIMITIVE_TYPE_NAMES:
-    type_id = getattr(CArrowType, type_name)
+    type_id = getattr(_types, type_name)
     _ITEMS_ITER_LOOKUP[type_id] = "_primitive_iter"
