@@ -15,11 +15,16 @@
 # specific language governing permissions and limitations
 # under the License.
 
+import array
+from datetime import date, datetime, timezone
+
 import pytest
-from nanoarrow._lib import CArrayBuilder, NanoarrowException
+from nanoarrow._array import CArrayBuilder
+from nanoarrow._utils import NanoarrowException
 from nanoarrow.c_schema import c_schema_view
 
 import nanoarrow as na
+from nanoarrow import device
 
 
 def test_c_array_from_c_array():
@@ -145,7 +150,7 @@ def test_c_array_shallow_copy():
     import gc
     import platform
 
-    from nanoarrow._lib import get_pyobject_buffer_count
+    from nanoarrow._utils import get_pyobject_buffer_count
 
     if platform.python_implementation() == "PyPy":
         pytest.skip(
@@ -283,6 +288,32 @@ def test_c_array_from_iterable_string():
         na.c_array([b"1234"], na.string())
 
 
+def test_c_array_from_iterable_string_view():
+    string = na.c_array(
+        ["abc", None, "a string longer than 12 bytes"], na.string_view()
+    )
+    assert string.length == 3
+    assert string.null_count == 1
+    assert string.n_buffers == 4
+
+    array_view = string.view()
+    assert len(array_view.buffer(0)) == 1
+    assert bytes(array_view.buffer(2)) == b"a string longer than 12 bytes"
+    assert list(array_view.buffer(3)) == [len("a string longer than 12 bytes")]
+
+    # Make sure this also works when all strings are inlined (i.e., no variadic buffers)
+    string = na.c_array(["abc", None, "short string"], na.string_view())
+    assert string.length == 3
+    assert string.null_count == 1
+    assert string.n_buffers == 3
+
+    array_view = string.view()
+    assert len(array_view.buffer(0)) == 1
+    assert len(array_view.buffer(1)) == 3
+    assert len(bytes(array_view.buffer(1))) == 3 * 16
+    assert list(array_view.buffer(2)) == []
+
+
 def test_c_array_from_iterable_bytes():
     string = na.c_array([b"abc", None, b"defg"], na.binary())
     assert string.length == 3
@@ -306,6 +337,20 @@ def test_c_array_from_iterable_bytes():
         na.c_array([buf_2d], na.binary())
 
 
+def test_c_array_from_iterable_view():
+    string = na.c_array(
+        [b"abc", None, b"a string longer than 12 bytes"], na.binary_view()
+    )
+    assert string.length == 3
+    assert string.null_count == 1
+    assert string.n_buffers == 4
+
+    array_view = string.view()
+    assert len(array_view.buffer(0)) == 1
+    assert bytes(array_view.buffer(2)) == b"a string longer than 12 bytes"
+    assert list(array_view.buffer(3)) == [len("a string longer than 12 bytes")]
+
+
 def test_c_array_from_iterable_non_empty_nullable_without_nulls():
     c_array = na.c_array([1, 2, 3], na.int32())
     assert c_array.length == 3
@@ -514,3 +559,187 @@ def test_c_array_from_buffers_validation():
             validation_level=validation_level,
         )
         assert c_array.length == 2
+
+
+def test_c_array_from_buffers_device():
+    c_device_array = na.c_array_from_buffers(
+        na.uint8(), 5, [None, b"12345"], device=device.cpu()
+    )
+    assert isinstance(c_device_array, device.CDeviceArray)
+
+    c_array = na.c_array(c_device_array)
+    assert c_array.length == 5
+    assert bytes(c_array.view().buffer(1)) == b"12345"
+
+
+def test_c_array_timestamp_seconds():
+    d1 = int(round(datetime(1970, 1, 1, tzinfo=timezone.utc).timestamp()))
+    d2 = int(round(datetime(1985, 12, 31, tzinfo=timezone.utc).timestamp()))
+    d3 = int(round(datetime(2005, 3, 4, tzinfo=timezone.utc).timestamp()))
+    c_array = na.c_array([d1, d2, d3], na.timestamp("s"))
+    assert c_array.length == 3
+    assert c_array.null_count == 0
+    view = c_array.view()
+    assert list(view.buffer(0)) == []
+    assert list(view.buffer(1)) == [d1, d2, d3]
+
+
+def test_c_array_timestamp_seconds_from_pybuffer():
+    d1 = int(round(datetime(1970, 1, 1, tzinfo=timezone.utc).timestamp()))
+    d2 = int(round(datetime(1985, 12, 31, tzinfo=timezone.utc).timestamp()))
+    d3 = int(round(datetime(2005, 3, 4, tzinfo=timezone.utc).timestamp()))
+    c_array = na.c_array(array.array("q", [d1, d2, d3]), na.timestamp("s"))
+    assert c_array.length == 3
+    assert c_array.null_count == 0
+    view = c_array.view()
+    assert list(view.buffer(0)) == []
+    assert list(view.buffer(1)) == [d1, d2, d3]
+
+
+def test_c_array_timestamp_milliseconds():
+    d1 = int(round(datetime(1970, 1, 1, tzinfo=timezone.utc).timestamp() * 1e3))
+    d2 = int(round(datetime(1985, 12, 31, tzinfo=timezone.utc).timestamp() * 1e3))
+    d3 = int(round(datetime(2005, 3, 4, tzinfo=timezone.utc).timestamp() * 1e3))
+    c_array = na.c_array([d1, d2, d3], na.timestamp("ms"))
+    assert c_array.length == 3
+    assert c_array.null_count == 0
+    view = c_array.view()
+    assert list(view.buffer(0)) == []
+    assert list(view.buffer(1)) == [d1, d2, d3]
+
+
+def test_c_array_timestamp_milliseconds_from_pybuffer():
+    d1 = int(round(datetime(1970, 1, 1, tzinfo=timezone.utc).timestamp() * 1e3))
+    d2 = int(round(datetime(1985, 12, 31, tzinfo=timezone.utc).timestamp() * 1e3))
+    d3 = int(round(datetime(2005, 3, 4, tzinfo=timezone.utc).timestamp() * 1e3))
+    c_array = na.c_array(array.array("q", [d1, d2, d3]), na.timestamp("ms"))
+    assert c_array.length == 3
+    assert c_array.null_count == 0
+    view = c_array.view()
+    assert list(view.buffer(0)) == []
+    assert list(view.buffer(1)) == [d1, d2, d3]
+
+
+def test_c_array_timestamp_microseconds():
+    d1 = int(round(datetime(1970, 1, 1, tzinfo=timezone.utc).timestamp() * 1e6))
+    d2 = int(round(datetime(1985, 12, 31, tzinfo=timezone.utc).timestamp() * 1e6))
+    d3 = int(round(datetime(2005, 3, 4, tzinfo=timezone.utc).timestamp() * 1e6))
+    c_array = na.c_array([d1, d2, d3], na.timestamp("us"))
+    assert c_array.length == 3
+    assert c_array.null_count == 0
+    view = c_array.view()
+    assert list(view.buffer(0)) == []
+    assert list(view.buffer(1)) == [d1, d2, d3]
+
+
+def test_c_array_timestamp_nanoseconds():
+    d1 = int(round(datetime(1970, 1, 1, tzinfo=timezone.utc).timestamp() * 1e9))
+    d2 = int(round(datetime(1985, 12, 31, tzinfo=timezone.utc).timestamp() * 1e9))
+    d3 = int(round(datetime(2005, 3, 4).timestamp() * 1e9))
+    c_array = na.c_array([d1, d2, d3], na.timestamp("ns"))
+    assert c_array.length == 3
+    assert c_array.null_count == 0
+    view = c_array.view()
+    assert list(view.buffer(0)) == []
+    assert list(view.buffer(1)) == [d1, d2, d3]
+
+
+def test_c_array_duration():
+    unix_epoch = date(1970, 1, 1)
+    d1, d2, d3 = date(1970, 1, 2), date(1970, 1, 3), date(1970, 1, 4)
+    d1_duration_in_ms = int(round((d1 - unix_epoch).total_seconds() * 1e3))
+    d2_duration_in_ms = int(round((d2 - unix_epoch).total_seconds() * 1e3))
+    d3_duration_in_ms = int(round((d3 - unix_epoch).total_seconds() * 1e3))
+    c_array = na.c_array(
+        [d1_duration_in_ms, d2_duration_in_ms, d3_duration_in_ms], na.duration("ms")
+    )
+    assert c_array.length == 3
+    assert c_array.null_count == 0
+    view = c_array.view()
+    assert list(view.buffer(0)) == []
+    assert list(view.buffer(1)) == [
+        d1_duration_in_ms,
+        d2_duration_in_ms,
+        d3_duration_in_ms,
+    ]
+
+
+def test_device_array_errors(cuda_device):
+    if not cuda_device:
+        pytest.skip("CUDA device not available")
+
+    # Check that we can't create a CUDA array from CPU buffers
+    with pytest.raises(ValueError, match="are not identical"):
+        na.c_array_from_buffers(
+            na.int64(),
+            3,
+            [None, na.c_buffer([1, 2, 3], na.int64())],
+            device=cuda_device,
+        )
+
+    # Check that we can't create a CUDA array from CPU children
+    with pytest.raises(ValueError, match="are not identical"):
+        na.c_array_from_buffers(
+            na.struct([na.int64()]),
+            length=0,
+            buffers=[None],
+            children=[na.c_array([], na.int64())],
+            device=cuda_device,
+        )
+
+
+def test_array_from_dlpack_cuda(cuda_device):
+    from nanoarrow.device import CDeviceArray, DeviceType
+
+    cp = pytest.importorskip("cupy")
+    if not cuda_device:
+        pytest.skip("CUDA device not available")
+
+    gpu_validity = cp.array([255], cp.uint8)
+    gpu_array = cp.array([1, 2, 3], cp.int64)
+
+    c_array = na.c_array_from_buffers(
+        na.int64(),
+        3,
+        [gpu_validity, gpu_array],
+        move=True,
+        device=cuda_device,
+    )
+    assert isinstance(c_array, CDeviceArray)
+    assert c_array.device_type == DeviceType.CUDA
+    assert c_array.device_id == 0
+
+    c_array_view = c_array.view()
+    assert c_array_view.storage_type == "int64"
+    assert c_array_view.buffer(0).device == cuda_device
+    assert c_array_view.buffer(1).device == cuda_device
+    assert len(c_array_view) == 3
+
+    # Make sure we don't attempt accessing a GPU buffer to calculate the null count
+    assert c_array_view.null_count == -1
+
+    # Check that buffers made it all the way through
+    cp.testing.assert_array_equal(cp.from_dlpack(c_array_view.buffer(1)), gpu_array)
+
+    # Also check a nested array
+    c_array_struct = na.c_array_from_buffers(
+        na.struct([na.int64()]),
+        3,
+        buffers=[None],
+        children=[c_array],
+        move=True,
+        device=cuda_device,
+    )
+    assert c_array_struct.device_type == DeviceType.CUDA
+    assert c_array_struct.device_id == 0
+
+    c_array_view_struct = c_array_struct.view()
+    assert c_array_view_struct.storage_type == "struct"
+    assert c_array_view_struct.buffer(0).device == cuda_device
+
+    c_array_view_child = c_array_view_struct.child(0)
+    assert c_array_view_child.buffer(0).device == cuda_device
+    assert c_array_view_child.buffer(1).device == cuda_device
+    cp.testing.assert_array_equal(
+        cp.from_dlpack(c_array_view_child.buffer(1)), gpu_array
+    )
