@@ -15,16 +15,21 @@
 # specific language governing permissions and limitations
 # under the License.
 
+import importlib
 import re
 
 import nanoarrow as na
 
 
 def test_version():
-    re_py_version = re.compile(r"^[0-9]+\.[0-9]+\.[0-9]+(\.dev[0-9+])?$")
+    re_py_version = re.compile(r"^[0-9]+\.[0-9]+\.[0-9]+(\.dev[0-9]+)?$")
     assert re_py_version.match(na.__version__) is not None
 
 
 def test_c_version():
     re_version = re.compile(r"^[0-9]+\.[0-9]+\.[0-9]+(-SNAPSHOT)?$")
     assert re_version.match(na.c_version()) is not None
+
+
+def test_metadata_version():
+    assert importlib.metadata.version("nanoarrow") == na.__version__
